@@ -174,6 +174,7 @@ pub use rsvg_internals::{
 /// functions to load an [`SvgHandle`][SvgHandle].
 ///
 /// [SvgHandle]: struct.SvgHandle.html
+#[derive(Default)]
 pub struct Loader {
     unlimited_size: bool,
     keep_image_data: bool,
@@ -200,10 +201,7 @@ impl Loader {
     ///     .unwrap();
     /// ```
     pub fn new() -> Self {
-        Loader {
-            unlimited_size: false,
-            keep_image_data: false,
-        }
+        Self::default()
     }
 
     /// Controls safety limits used in the XML parser.
@@ -517,7 +515,7 @@ impl<'a> CairoRenderer<'a> {
         self.handle
             .0
             .get_geometry_for_layer(id, viewport, self.dpi, false)
-            .map(|(i, l)| (i.into(), l.into()))
+            .map(|(i, l)| (i, l))
     }
 
     /// Renders a single SVG element in the same place as for a whole SVG document
@@ -587,7 +585,7 @@ impl<'a> CairoRenderer<'a> {
         self.handle
             .0
             .get_geometry_for_element(id, self.dpi, false)
-            .map(|(i, l)| (i.into(), l.into()))
+            .map(|(i, l)| (i, l))
     }
 
     /// Renders a single SVG element to a given viewport
