@@ -1,6 +1,8 @@
 //! The main context structure which drives the drawing process.
 
+use float_cmp::approx_eq;
 use once_cell::sync::Lazy;
+use pango::FontMapExt;
 use regex::{Captures, Regex};
 use std::borrow::Cow;
 use std::cell::RefCell;
@@ -17,14 +19,17 @@ use crate::dpi::Dpi;
 use crate::element::Element;
 use crate::error::{AcquireError, RenderingError};
 use crate::filters;
+use crate::float_eq_cairo::ApproxEqCairo;
+use crate::gradient::{Gradient, GradientUnits, GradientVariant, SpreadMethod};
 use crate::marker;
 use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::paint_server::{PaintServer, PaintSource};
 use crate::path_builder::*;
+use crate::pattern::{PatternContentUnits, PatternUnits, ResolvedPattern};
 use crate::properties::ComputedValues;
 use crate::property_defs::{
-    ClipRule, FillRule, Opacity, Overflow, ShapeRendering, StrokeDasharray, StrokeLinecap,
-    StrokeLinejoin,
+    ClipRule, FillRule, MixBlendMode, Opacity, Overflow, PaintTarget, ShapeRendering,
+    StrokeDasharray, StrokeLinecap, StrokeLinejoin, TextRendering,
 };
 use crate::rect::Rect;
 use crate::shapes::Markers;
@@ -165,15 +170,7 @@ impl DrawingCtx {
         self.measuring
     }
 
-    pub fn is_testing(&self) -> bool {
-        self.testing
-    }
-
-    pub fn get_cairo_context(&self) -> cairo::Context {
-        self.cr.clone()
-    }
-
-    pub fn get_transform(&self) -> Transform {
+    fn get_transform(&self) -> Transform {
         Transform::from(self.cr.get_matrix())
     }
 
@@ -186,7 +183,7 @@ impl DrawingCtx {
     // It would be better to have an explicit push/pop for the cairo_t, or
     // pushing a temporary surface, or something that does not involve
     // monkeypatching the cr directly.
-    pub fn set_cairo_context(&mut self, cr: &cairo::Context) {
+    fn set_cairo_context(&mut self, cr: &cairo::Context) {
         self.cr = cr.clone();
     }
 
@@ -282,7 +279,7 @@ impl DrawingCtx {
         preserve_aspect_ratio: AspectRatio,
         clip_mode: Option<ClipMode>,
     ) -> Option<ViewParams> {
-        let cr = self.get_cairo_context();
+        let cr = self.cr.clone();
 
         if let Some(ClipMode::ClipToViewport) = clip_mode {
             cr.rectangle(
@@ -350,7 +347,7 @@ impl DrawingCtx {
             let cascaded = CascadedValues::new_from_node(node);
 
             self.with_saved_transform(Some(node_transform), &mut |dc| {
-                let cr = dc.get_cairo_context();
+                let cr = dc.cr.clone();
 
                 // here we don't push a layer because we are clipping
                 let res = node.draw_children(acquired_nodes, &cascaded, dc, true);
@@ -510,10 +507,11 @@ impl DrawingCtx {
                 // Here we are clipping in user space, so the bbox doesn't matter
                 dc.clip_to_node(&clip_in_user_space, acquired_nodes, &dc.empty_bbox())?;
 
-                let is_opaque = (opacity - 1.0).abs() < f64::EPSILON;
+                let is_opaque = approx_eq!(f64, opacity, 1.0);
                 let needs_temporary_surface = !(is_opaque
                     && filter.is_none()
                     && mask.is_none()
+                    && values.mix_blend_mode() == MixBlendMode::Normal
                     && clip_in_object_space.is_none());
 
                 if needs_temporary_surface {
@@ -632,6 +630,7 @@ impl DrawingCtx {
                         // No mask, so composite the temporary surface
 
                         dc.cr.set_matrix(affines.compositing.into());
+                        dc.cr.set_operator(values.mix_blend_mode().into());
 
                         if opacity < 1.0 {
                             dc.cr.paint_with_alpha(opacity);
@@ -729,7 +728,7 @@ impl DrawingCtx {
     }
 
     /// Saves the current Cairo context, runs the draw_fn, and restores the context
-    pub fn with_saved_cr(
+    fn with_saved_cr(
         &mut self,
         draw_fn: &mut dyn FnMut(&mut DrawingCtx) -> Result<BoundingBox, RenderingError>,
     ) -> Result<BoundingBox, RenderingError> {
@@ -749,7 +748,7 @@ impl DrawingCtx {
 
         let attributes = format!("uri='{}'", escape_link_target(link_target));
 
-        let cr = self.get_cairo_context();
+        let cr = self.cr.clone();
         cr.tag_begin(CAIRO_TAG_LINK, &attributes);
 
         let res = draw_fn(self);
@@ -780,6 +779,7 @@ impl DrawingCtx {
                             child_surface,
                             acquired_nodes,
                             self,
+                            self.get_transform(),
                             node_bbox,
                         );
                     }
@@ -805,103 +805,337 @@ impl DrawingCtx {
         Ok(child_surface)
     }
 
+    fn set_gradient(
+        self: &mut DrawingCtx,
+        gradient: &Gradient,
+        _acquired_nodes: &mut AcquiredNodes,
+        opacity: UnitInterval,
+        values: &ComputedValues,
+        bbox: &BoundingBox,
+    ) -> Result<bool, RenderingError> {
+        let units = gradient.get_units();
+        let transform = if let Ok(t) = bbox.rect_to_transform(units.0) {
+            t
+        } else {
+            return Ok(false);
+        };
+
+        let params = if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
+            self.push_view_box(1.0, 1.0)
+        } else {
+            self.get_view_params()
+        };
+
+        let g = match gradient.get_variant() {
+            GradientVariant::Linear { x1, y1, x2, y2 } => {
+                cairo::Gradient::clone(&cairo::LinearGradient::new(
+                    x1.normalize(values, &params),
+                    y1.normalize(values, &params),
+                    x2.normalize(values, &params),
+                    y2.normalize(values, &params),
+                ))
+            }
+
+            GradientVariant::Radial {
+                cx,
+                cy,
+                r,
+                fx,
+                fy,
+                fr,
+            } => {
+                let n_cx = cx.normalize(values, &params);
+                let n_cy = cy.normalize(values, &params);
+                let n_r = r.normalize(values, &params);
+                let n_fx = fx.normalize(values, &params);
+                let n_fy = fy.normalize(values, &params);
+                let n_fr = fr.normalize(values, &params);
+
+                cairo::Gradient::clone(&cairo::RadialGradient::new(
+                    n_fx, n_fy, n_fr, n_cx, n_cy, n_r,
+                ))
+            }
+        };
+
+        let transform = transform.pre_transform(&gradient.get_transform());
+        if let Some(m) = transform.invert() {
+            g.set_matrix(m.into())
+        }
+
+        g.set_extend(cairo::Extend::from(gradient.get_spread()));
+
+        for stop in gradient.get_stops() {
+            let UnitInterval(stop_offset) = stop.offset;
+            let UnitInterval(o) = opacity;
+            let UnitInterval(stop_opacity) = stop.opacity;
+
+            g.add_color_stop_rgba(
+                stop_offset,
+                f64::from(stop.rgba.red_f32()),
+                f64::from(stop.rgba.green_f32()),
+                f64::from(stop.rgba.blue_f32()),
+                f64::from(stop.rgba.alpha_f32()) * stop_opacity * o,
+            );
+        }
+
+        let cr = self.cr.clone();
+        cr.set_source(&g);
+
+        Ok(true)
+    }
+
+    fn set_pattern(
+        &mut self,
+        pattern: &ResolvedPattern,
+        acquired_nodes: &mut AcquiredNodes,
+        opacity: UnitInterval,
+        values: &ComputedValues,
+        bbox: &BoundingBox,
+    ) -> Result<bool, RenderingError> {
+        let node = if let Some(n) = pattern.node_with_children() {
+            n
+        } else {
+            // This means we didn't find any children among the fallbacks,
+            // so there is nothing to render.
+            return Ok(false);
+        };
+
+        let units = pattern.get_units();
+        let content_units = pattern.get_content_units();
+        let pattern_transform = pattern.get_transform();
+
+        let params = if units == PatternUnits(CoordUnits::ObjectBoundingBox) {
+            self.push_view_box(1.0, 1.0)
+        } else {
+            self.get_view_params()
+        };
+
+        let pattern_rect = pattern.get_rect(values, &params);
+
+        // Work out the size of the rectangle so it takes into account the object bounding box
+        let (bbwscale, bbhscale) = match units {
+            PatternUnits(CoordUnits::ObjectBoundingBox) => bbox.rect.unwrap().size(),
+            PatternUnits(CoordUnits::UserSpaceOnUse) => (1.0, 1.0),
+        };
+
+        let taffine = self.get_transform().pre_transform(&pattern_transform);
+
+        let mut scwscale = (taffine.xx.powi(2) + taffine.xy.powi(2)).sqrt();
+        let mut schscale = (taffine.yx.powi(2) + taffine.yy.powi(2)).sqrt();
+
+        let scaled_width = pattern_rect.width() * bbwscale;
+        let scaled_height = pattern_rect.height() * bbhscale;
+
+        if approx_eq!(f64, scaled_width, 0.0) || approx_eq!(f64, scaled_height, 0.0) {
+            return Ok(false);
+        }
+
+        let pw: i32 = (scaled_width * scwscale) as i32;
+        let ph: i32 = (scaled_height * schscale) as i32;
+
+        if pw < 1 || ph < 1 {
+            return Ok(false);
+        }
+
+        scwscale = f64::from(pw) / scaled_width;
+        schscale = f64::from(ph) / scaled_height;
+
+        // Create the pattern coordinate system
+        let mut affine = match units {
+            PatternUnits(CoordUnits::ObjectBoundingBox) => {
+                let bbrect = bbox.rect.unwrap();
+                Transform::new_translate(
+                    bbrect.x0 + pattern_rect.x0 * bbrect.width(),
+                    bbrect.y0 + pattern_rect.y0 * bbrect.height(),
+                )
+            }
+
+            PatternUnits(CoordUnits::UserSpaceOnUse) => {
+                Transform::new_translate(pattern_rect.x0, pattern_rect.y0)
+            }
+        };
+
+        // Apply the pattern transform
+        affine = affine.post_transform(&pattern_transform);
+
+        let mut caffine: Transform;
+
+        // Create the pattern contents coordinate system
+        let _params = if let Some(vbox) = pattern.get_vbox() {
+            // If there is a vbox, use that
+            let r = pattern
+                .get_preserve_aspect_ratio()
+                .compute(&vbox, &Rect::from_size(scaled_width, scaled_height));
+
+            let sw = r.width() / vbox.0.width();
+            let sh = r.height() / vbox.0.height();
+            let x = r.x0 - vbox.0.x0 * sw;
+            let y = r.y0 - vbox.0.y0 * sh;
+
+            caffine = Transform::new_scale(sw, sh).pre_translate(x, y);
+
+            self.push_view_box(vbox.0.width(), vbox.0.height())
+        } else if content_units == PatternContentUnits(CoordUnits::ObjectBoundingBox) {
+            // If coords are in terms of the bounding box, use them
+            let (bbw, bbh) = bbox.rect.unwrap().size();
+
+            caffine = Transform::new_scale(bbw, bbh);
+
+            self.push_view_box(1.0, 1.0)
+        } else {
+            caffine = Transform::identity();
+            self.get_view_params()
+        };
+
+        if !scwscale.approx_eq_cairo(1.0) || !schscale.approx_eq_cairo(1.0) {
+            caffine = caffine.post_scale(scwscale, schscale);
+            affine = affine.pre_scale(1.0 / scwscale, 1.0 / schscale);
+        }
+
+        // Draw to another surface
+
+        let cr_save = self.cr.clone();
+
+        let surface = cr_save
+            .get_target()
+            .create_similar(cairo::Content::ColorAlpha, pw, ph)?;
+
+        let cr_pattern = cairo::Context::new(&surface);
+
+        self.set_cairo_context(&cr_pattern);
+
+        // Set up transformations to be determined by the contents units
+        cr_pattern.set_matrix(caffine.into());
+
+        // Draw everything
+        let res = self.with_alpha(opacity, &mut |dc| {
+            let pattern_cascaded = CascadedValues::new_from_node(&node);
+            let pattern_values = pattern_cascaded.get();
+            dc.with_discrete_layer(
+                &node,
+                acquired_nodes,
+                pattern_values,
+                false,
+                &mut |an, dc| node.draw_children(an, &pattern_cascaded, dc, false),
+            )
+        });
+
+        // Return to the original coordinate system and rendering context
+        self.set_cairo_context(&cr_save);
+
+        // Set the final surface as a Cairo pattern into the Cairo context
+        let pattern = cairo::SurfacePattern::create(&surface);
+
+        if let Some(m) = affine.invert() {
+            pattern.set_matrix(m.into())
+        }
+        pattern.set_extend(cairo::Extend::Repeat);
+        pattern.set_filter(cairo::Filter::Best);
+        cr_save.set_source(&pattern);
+
+        res.map(|_| true)
+    }
+
     fn set_color(
         &self,
         color: cssparser::Color,
         opacity: UnitInterval,
         current_color: cssparser::RGBA,
-    ) {
+    ) -> Result<bool, RenderingError> {
         let rgba = match color {
             cssparser::Color::RGBA(rgba) => rgba,
             cssparser::Color::CurrentColor => current_color,
         };
 
         let UnitInterval(o) = opacity;
-        self.get_cairo_context().set_source_rgba(
+        self.cr.clone().set_source_rgba(
             f64::from(rgba.red_f32()),
             f64::from(rgba.green_f32()),
             f64::from(rgba.blue_f32()),
             f64::from(rgba.alpha_f32()) * o,
         );
+
+        Ok(true)
     }
 
-    pub fn set_source_paint_server(
+    fn set_source_paint_server(
         &mut self,
         acquired_nodes: &mut AcquiredNodes,
-        ps: &PaintServer,
+        paint_server: &PaintServer,
         opacity: UnitInterval,
         bbox: &BoundingBox,
         current_color: cssparser::RGBA,
+        values: &ComputedValues,
     ) -> Result<bool, RenderingError> {
-        match *ps {
-            PaintServer::Iri {
-                ref iri,
-                ref alternate,
-            } => {
-                let mut had_paint_server = false;
+        let paint_source = paint_server.resolve(acquired_nodes)?;
 
-                match acquired_nodes.acquire(iri) {
-                    Ok(acquired) => {
-                        let node = acquired.get();
-
-                        assert!(node.is_element());
-
-                        had_paint_server = match *node.borrow_element() {
-                            Element::LinearGradient(ref g) => g.resolve_fallbacks_and_set_pattern(
-                                &node,
-                                acquired_nodes,
-                                self,
-                                opacity,
-                                bbox,
-                            )?,
-                            Element::RadialGradient(ref g) => g.resolve_fallbacks_and_set_pattern(
-                                &node,
-                                acquired_nodes,
-                                self,
-                                opacity,
-                                bbox,
-                            )?,
-                            Element::Pattern(ref p) => p.resolve_fallbacks_and_set_pattern(
-                                &node,
-                                acquired_nodes,
-                                self,
-                                opacity,
-                                bbox,
-                            )?,
-                            _ => false,
-                        }
-                    }
-
-                    Err(AcquireError::MaxReferencesExceeded) => {
-                        return Err(RenderingError::InstancingLimit);
-                    }
-
-                    Err(_) => (),
-                }
-
-                if !had_paint_server && alternate.is_some() {
-                    self.set_color(alternate.unwrap(), opacity, current_color);
-                    had_paint_server = true;
+        match paint_source {
+            PaintSource::Gradient(g, c) => {
+                if self.set_gradient(&g, acquired_nodes, opacity, values, bbox)? {
+                    Ok(true)
+                } else if let Some(c) = c {
+                    self.set_color(c, opacity, current_color)
                 } else {
-                    rsvg_log!(
-                        "pattern \"{}\" was not found and there was no fallback alternate",
-                        iri
-                    );
+                    Ok(false)
                 }
-
-                Ok(had_paint_server)
             }
-
-            PaintServer::SolidColor(color) => {
-                self.set_color(color, opacity, current_color);
-                Ok(true)
+            PaintSource::Pattern(p, c) => {
+                if self.set_pattern(&p, acquired_nodes, opacity, values, bbox)? {
+                    Ok(true)
+                } else if let Some(c) = c {
+                    self.set_color(c, opacity, current_color)
+                } else {
+                    Ok(false)
+                }
             }
-
-            PaintServer::None => Ok(false),
+            PaintSource::SolidColor(c) => self.set_color(c, opacity, current_color),
+            PaintSource::None => Ok(false),
         }
     }
 
-    pub fn setup_cr_for_stroke(&self, cr: &cairo::Context, values: &ComputedValues) {
+    /// Computes and returns a surface corresponding to the given paint server.
+    pub fn get_paint_server_surface(
+        &mut self,
+        width: i32,
+        height: i32,
+        acquired_nodes: &mut AcquiredNodes,
+        paint_server: &PaintServer,
+        opacity: UnitInterval,
+        bbox: &BoundingBox,
+        current_color: cssparser::RGBA,
+        values: &ComputedValues,
+    ) -> Result<SharedImageSurface, cairo::Status> {
+        let mut surface = ExclusiveImageSurface::new(width, height, SurfaceType::SRgb)?;
+
+        surface.draw(&mut |cr| {
+            let cr_save = self.cr.clone();
+            self.set_cairo_context(&cr);
+
+            // FIXME: we are ignoring any error
+            let _ = self
+                .set_source_paint_server(
+                    acquired_nodes,
+                    paint_server,
+                    opacity,
+                    bbox,
+                    current_color,
+                    values,
+                )
+                .map(|had_paint_server| {
+                    if had_paint_server {
+                        cr.paint();
+                    }
+                });
+
+            self.set_cairo_context(&cr_save);
+
+            Ok(())
+        })?;
+
+        surface.share()
+    }
+
+    fn setup_cr_for_stroke(&self, cr: &cairo::Context, values: &ComputedValues) {
         let params = self.get_view_params();
 
         cr.set_line_width(values.stroke_width().0.normalize(values, &params));
@@ -926,61 +1160,54 @@ impl DrawingCtx {
         }
     }
 
-    pub fn stroke_and_fill(
+    fn stroke(
         &mut self,
         cr: &cairo::Context,
         acquired_nodes: &mut AcquiredNodes,
         values: &ComputedValues,
-    ) -> Result<BoundingBox, RenderingError> {
-        cr.set_antialias(cairo::Antialias::from(values.shape_rendering()));
+        bbox: &BoundingBox,
+        current_color: cssparser::RGBA,
+    ) -> Result<(), RenderingError> {
+        self.set_source_paint_server(
+            acquired_nodes,
+            &values.stroke().0,
+            values.stroke_opacity().0,
+            bbox,
+            current_color,
+            values,
+        )
+        .map(|had_paint_server| {
+            if had_paint_server {
+                cr.stroke_preserve();
+            }
+        })?;
 
-        self.setup_cr_for_stroke(cr, values);
+        Ok(())
+    }
 
-        // Update the bbox in the rendering context.  Below, we actually set the
-        // fill/stroke patterns on the cairo_t.  That process requires the
-        // rendering context to have an updated bbox; for example, for the
-        // coordinate system in patterns.
-        let bbox = compute_stroke_and_fill_box(cr, values);
+    fn fill(
+        &mut self,
+        cr: &cairo::Context,
+        acquired_nodes: &mut AcquiredNodes,
+        values: &ComputedValues,
+        bbox: &BoundingBox,
+        current_color: cssparser::RGBA,
+    ) -> Result<(), RenderingError> {
+        self.set_source_paint_server(
+            acquired_nodes,
+            &values.fill().0,
+            values.fill_opacity().0,
+            bbox,
+            current_color,
+            values,
+        )
+        .map(|had_paint_server| {
+            if had_paint_server {
+                cr.fill_preserve();
+            }
+        })?;
 
-        let current_color = values.color().0;
-
-        let res = self
-            .set_source_paint_server(
-                acquired_nodes,
-                &values.fill().0,
-                values.fill_opacity().0,
-                &bbox,
-                current_color,
-            )
-            .map(|had_paint_server| {
-                if had_paint_server {
-                    if values.stroke().0 == PaintServer::None {
-                        cr.fill();
-                    } else {
-                        cr.fill_preserve();
-                    }
-                }
-            })
-            .and_then(|_| {
-                self.set_source_paint_server(
-                    acquired_nodes,
-                    &values.stroke().0,
-                    values.stroke_opacity().0,
-                    &bbox,
-                    current_color,
-                )
-                .map(|had_paint_server| {
-                    if had_paint_server {
-                        cr.stroke();
-                    }
-                })
-            });
-
-        // clear the path in case stroke == fill == None; otherwise
-        // we leave it around from computing the bounding box
-        cr.new_path();
-
-        res.map(|_: ()| bbox)
+        Ok(())
     }
 
     pub fn draw_path(
@@ -992,31 +1219,220 @@ impl DrawingCtx {
         markers: Markers,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
-        if !path.is_empty() {
-            let bbox =
-                self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
-                    let cr = dc.get_cairo_context();
+        if path.is_empty() {
+            return Ok(self.empty_bbox());
+        }
 
-                    let is_square_linecap = values.stroke_line_cap() == StrokeLinecap::Square;
-                    path.to_cairo(&cr, is_square_linecap)?;
+        self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
+            let cr = dc.cr.clone();
+            let is_square_linecap = values.stroke_line_cap() == StrokeLinecap::Square;
 
-                    if clipping {
-                        cr.set_fill_rule(cairo::FillRule::from(values.clip_rule()));
-                        Ok(dc.empty_bbox())
-                    } else {
-                        cr.set_fill_rule(cairo::FillRule::from(values.fill_rule()));
-                        dc.stroke_and_fill(&cr, an, values)
-                    }
-                })?;
-
-            if markers == Markers::Yes {
-                marker::render_markers_for_path(path, self, acquired_nodes, values, clipping)?;
+            if clipping {
+                cr.set_fill_rule(cairo::FillRule::from(values.clip_rule()));
+                path.to_cairo(&cr, is_square_linecap)?;
+                return Ok(dc.empty_bbox());
             }
 
-            Ok(bbox)
-        } else {
-            Ok(self.empty_bbox())
+            let current_color = values.color().0;
+
+            cr.set_antialias(cairo::Antialias::from(values.shape_rendering()));
+            dc.setup_cr_for_stroke(&cr, values);
+
+            cr.set_fill_rule(cairo::FillRule::from(values.fill_rule()));
+
+            let mut bounding_box: Option<BoundingBox> = None;
+            let mut has_path = false;
+            cr.new_path();
+
+            for &target in &values.paint_order().targets {
+                // fill and stroke operations will preserve the path.
+                // markers operation will clear the path.
+                match target {
+                    PaintTarget::Fill | PaintTarget::Stroke => {
+                        if !has_path {
+                            path.to_cairo(&cr, is_square_linecap)?;
+                            has_path = true;
+                        }
+                        let bbox = bounding_box
+                            .get_or_insert_with(|| compute_stroke_and_fill_box(&cr, &values));
+
+                        if target == PaintTarget::Stroke {
+                            dc.stroke(&cr, an, values, &bbox, current_color)?;
+                        } else {
+                            dc.fill(&cr, an, values, &bbox, current_color)?;
+                        }
+                    }
+                    PaintTarget::Markers if markers == Markers::Yes => {
+                        if has_path {
+                            cr.new_path();
+                            has_path = false;
+                        }
+                        marker::render_markers_for_path(path, dc, an, values, clipping)?;
+                    }
+                    _ => {}
+                }
+            }
+
+            if has_path {
+                cr.new_path();
+            }
+
+            Ok(bounding_box.unwrap())
+        })
+    }
+
+    pub fn draw_image(
+        &mut self,
+        surface: &SharedImageSurface,
+        rect: Rect,
+        aspect: AspectRatio,
+        node: &Node,
+        acquired_nodes: &mut AcquiredNodes,
+        values: &ComputedValues,
+        clipping: bool,
+    ) -> Result<BoundingBox, RenderingError> {
+        let image_width = surface.width();
+        let image_height = surface.height();
+        if clipping || rect.is_empty() || image_width == 0 || image_height == 0 {
+            return Ok(self.empty_bbox());
         }
+
+        let image_width = f64::from(image_width);
+        let image_height = f64::from(image_height);
+        let vbox = ViewBox(Rect::from_size(image_width, image_height));
+
+        let clip_mode = if !values.is_overflow() && aspect.is_slice() {
+            Some(ClipMode::ClipToViewport)
+        } else {
+            None
+        };
+
+        self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |_an, dc| {
+            dc.with_saved_cr(&mut |dc| {
+                if let Some(_params) = dc.push_new_viewport(Some(vbox), rect, aspect, clip_mode) {
+                    let cr = dc.cr.clone();
+
+                    // We need to set extend appropriately, so can't use cr.set_source_surface().
+                    //
+                    // If extend is left at its default value (None), then bilinear scaling uses
+                    // transparency outside of the image producing incorrect results.
+                    // For example, in svg1.1/filters-blend-01-b.svgthere's a completely
+                    // opaque 100×1 image of a gradient scaled to 100×98 which ends up
+                    // transparent almost everywhere without this fix (which it shouldn't).
+                    let ptn = surface.to_cairo_pattern();
+                    ptn.set_extend(cairo::Extend::Pad);
+                    cr.set_source(&ptn);
+
+                    // Clip is needed due to extend being set to pad.
+                    cr.rectangle(0.0, 0.0, image_width, image_height);
+                    cr.clip();
+
+                    cr.paint();
+                }
+
+                // The bounding box for <image> is decided by the values of x, y, w, h
+                // and not by the final computed image bounds.
+                Ok(dc.empty_bbox().with_rect(rect))
+            })
+        })
+    }
+
+    pub fn draw_text(
+        &mut self,
+        layout: &pango::Layout,
+        x: f64,
+        y: f64,
+        acquired_nodes: &mut AcquiredNodes,
+        values: &ComputedValues,
+        clipping: bool,
+    ) -> Result<BoundingBox, RenderingError> {
+        let transform = self.get_transform();
+
+        let gravity = layout.get_context().unwrap().get_gravity();
+
+        let bbox = compute_text_box(layout, x, y, transform, gravity);
+        if bbox.is_none() {
+            return Ok(self.empty_bbox());
+        }
+
+        let mut bbox = if clipping {
+            self.empty_bbox()
+        } else {
+            bbox.unwrap()
+        };
+
+        self.with_saved_cr(&mut |dc| {
+            let cr = dc.cr.clone();
+
+            cr.set_antialias(cairo::Antialias::from(values.text_rendering()));
+            dc.setup_cr_for_stroke(&cr, &values);
+            cr.move_to(x, y);
+
+            let rotation = gravity.to_rotation();
+            if !rotation.approx_eq_cairo(0.0) {
+                cr.rotate(-rotation);
+            }
+
+            let current_color = values.color().0;
+
+            let res = if !clipping {
+                dc.set_source_paint_server(
+                    acquired_nodes,
+                    &values.fill().0,
+                    values.fill_opacity().0,
+                    &bbox,
+                    current_color,
+                    values,
+                )
+                .map(|had_paint_server| {
+                    if had_paint_server {
+                        pangocairo::functions::update_layout(&cr, &layout);
+                        pangocairo::functions::show_layout(&cr, &layout);
+                    };
+                })
+            } else {
+                Ok(())
+            };
+
+            if res.is_ok() {
+                let mut need_layout_path = clipping;
+
+                let res = if !clipping {
+                    dc.set_source_paint_server(
+                        acquired_nodes,
+                        &values.stroke().0,
+                        values.stroke_opacity().0,
+                        &bbox,
+                        current_color,
+                        values,
+                    )
+                    .map(|had_paint_server| {
+                        if had_paint_server {
+                            need_layout_path = true;
+                        }
+                    })
+                } else {
+                    Ok(())
+                };
+
+                if res.is_ok() && need_layout_path {
+                    pangocairo::functions::update_layout(&cr, &layout);
+                    pangocairo::functions::layout_path(&cr, &layout);
+
+                    if !clipping {
+                        let (x0, y0, x1, y1) = cr.stroke_extents();
+                        let r = Rect::new(x0, y0, x1, y1);
+                        let ib = BoundingBox::new()
+                            .with_transform(transform)
+                            .with_ink_rect(r);
+                        cr.stroke();
+                        bbox.insert(&ib);
+                    }
+                }
+            }
+
+            res.map(|_: ()| bbox)
+        })
     }
 
     pub fn get_snapshot(
@@ -1219,7 +1635,7 @@ impl DrawingCtx {
         };
 
         // all other nodes
-        let cr = self.get_cairo_context();
+        let cr = self.cr.clone();
         cr.translate(use_rect.x0, use_rect.y0);
 
         self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
@@ -1357,6 +1773,59 @@ fn compute_stroke_and_fill_box(cr: &cairo::Context, values: &ComputedValues) ->
     bbox
 }
 
+fn compute_text_box(
+    layout: &pango::Layout,
+    x: f64,
+    y: f64,
+    transform: Transform,
+    gravity: pango::Gravity,
+) -> Option<BoundingBox> {
+    #![allow(clippy::many_single_char_names)]
+
+    let (ink, _) = layout.get_extents();
+    if ink.width == 0 || ink.height == 0 {
+        return None;
+    }
+
+    let ink_x = f64::from(ink.x);
+    let ink_y = f64::from(ink.y);
+    let ink_width = f64::from(ink.width);
+    let ink_height = f64::from(ink.height);
+    let pango_scale = f64::from(pango::SCALE);
+
+    let (x, y, w, h) = if gravity_is_vertical(gravity) {
+        (
+            x + (ink_x - ink_height) / pango_scale,
+            y + ink_y / pango_scale,
+            ink_height / pango_scale,
+            ink_width / pango_scale,
+        )
+    } else {
+        (
+            x + ink_x / pango_scale,
+            y + ink_y / pango_scale,
+            ink_width / pango_scale,
+            ink_height / pango_scale,
+        )
+    };
+
+    let r = Rect::new(x, y, x + w, y + h);
+    let bbox = BoundingBox::new()
+        .with_transform(transform)
+        .with_rect(r)
+        .with_ink_rect(r);
+
+    Some(bbox)
+}
+
+// FIXME: should the pango crate provide this like PANGO_GRAVITY_IS_VERTICAL() ?
+fn gravity_is_vertical(gravity: pango::Gravity) -> bool {
+    match gravity {
+        pango::Gravity::East | pango::Gravity::West => true,
+        _ => false,
+    }
+}
+
 /// escape quotes and backslashes with backslash
 fn escape_link_target(value: &str) -> Cow<'_, str> {
     static REGEX: Lazy<Regex> = Lazy::new(|| Regex::new(r"['\\]").unwrap());
@@ -1370,6 +1839,16 @@ fn escape_link_target(value: &str) -> Cow<'_, str> {
     })
 }
 
+impl From<SpreadMethod> for cairo::Extend {
+    fn from(s: SpreadMethod) -> cairo::Extend {
+        match s {
+            SpreadMethod::Pad => cairo::Extend::Pad,
+            SpreadMethod::Reflect => cairo::Extend::Reflect,
+            SpreadMethod::Repeat => cairo::Extend::Repeat,
+        }
+    }
+}
+
 impl From<StrokeLinejoin> for cairo::LineJoin {
     fn from(j: StrokeLinejoin) -> cairo::LineJoin {
         match j {
@@ -1390,6 +1869,31 @@ impl From<StrokeLinecap> for cairo::LineCap {
     }
 }
 
+impl From<MixBlendMode> for cairo::Operator {
+    fn from(m: MixBlendMode) -> cairo::Operator {
+        use cairo::Operator;
+
+        match m {
+            MixBlendMode::Normal => Operator::Over,
+            MixBlendMode::Multiply => Operator::Multiply,
+            MixBlendMode::Screen => Operator::Screen,
+            MixBlendMode::Overlay => Operator::Overlay,
+            MixBlendMode::Darken => Operator::Darken,
+            MixBlendMode::Lighten => Operator::Lighten,
+            MixBlendMode::ColorDodge => Operator::ColorDodge,
+            MixBlendMode::ColorBurn => Operator::ColorBurn,
+            MixBlendMode::HardLight => Operator::HardLight,
+            MixBlendMode::SoftLight => Operator::SoftLight,
+            MixBlendMode::Difference => Operator::Difference,
+            MixBlendMode::Exclusion => Operator::Exclusion,
+            MixBlendMode::Hue => Operator::HslHue,
+            MixBlendMode::Saturation => Operator::HslSaturation,
+            MixBlendMode::Color => Operator::HslColor,
+            MixBlendMode::Luminosity => Operator::HslLuminosity,
+        }
+    }
+}
+
 impl From<ClipRule> for cairo::FillRule {
     fn from(c: ClipRule) -> cairo::FillRule {
         match c {
@@ -1416,3 +1920,53 @@ impl From<ShapeRendering> for cairo::Antialias {
         }
     }
 }
+
+impl From<TextRendering> for cairo::Antialias {
+    fn from(tr: TextRendering) -> cairo::Antialias {
+        match tr {
+            TextRendering::Auto
+            | TextRendering::OptimizeLegibility
+            | TextRendering::GeometricPrecision => cairo::Antialias::Default,
+            TextRendering::OptimizeSpeed => cairo::Antialias::None,
+        }
+    }
+}
+
+impl From<&DrawingCtx> for pango::Context {
+    fn from(draw_ctx: &DrawingCtx) -> pango::Context {
+        let cr = draw_ctx.cr.clone();
+        let font_map = pangocairo::FontMap::get_default().unwrap();
+        let context = font_map.create_context().unwrap();
+        pangocairo::functions::update_context(&cr, &context);
+
+        // Pango says this about pango_cairo_context_set_resolution():
+        //
+        //     Sets the resolution for the context. This is a scale factor between
+        //     points specified in a #PangoFontDescription and Cairo units. The
+        //     default value is 96, meaning that a 10 point font will be 13
+        //     units high. (10 * 96. / 72. = 13.3).
+        //
+        // I.e. Pango font sizes in a PangoFontDescription are in *points*, not pixels.
+        // However, we are normalizing everything to userspace units, which amount to
+        // pixels.  So, we will use 72.0 here to make Pango not apply any further scaling
+        // to the size values we give it.
+        //
+        // An alternative would be to divide our font sizes by (dpi_y / 72) to effectively
+        // cancel out Pango's scaling, but it's probably better to deal with Pango-isms
+        // right here, instead of spreading them out through our Length normalization
+        // code.
+        pangocairo::functions::context_set_resolution(&context, 72.0);
+
+        if draw_ctx.testing {
+            let mut options = cairo::FontOptions::new();
+
+            options.set_antialias(cairo::Antialias::Gray);
+            options.set_hint_style(cairo::HintStyle::Full);
+            options.set_hint_metrics(cairo::HintMetrics::On);
+
+            pangocairo::functions::context_set_font_options(&context, Some(&options));
+        }
+
+        context
+    }
+}
