@@ -1,7 +1,6 @@
 //! Text elements: `text`, `tspan`, `tref`.
 
 use markup5ever::{expanded_name, local_name, namespace_url, ns};
-use pango::FontMapExt;
 use std::cell::RefCell;
 
 use crate::allowed_url::Fragment;
@@ -10,7 +9,6 @@ use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
-use crate::float_eq_cairo::ApproxEqCairo;
 use crate::font_props::FontWeight;
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow};
@@ -18,12 +16,10 @@ use crate::parsers::ParseValue;
 use crate::properties::ComputedValues;
 use crate::property_bag::PropertyBag;
 use crate::property_defs::{
-    Direction, FontStretch, FontStyle, FontVariant, TextAnchor, TextRendering, UnicodeBidi,
-    WritingMode, XmlLang, XmlSpace,
+    Direction, FontStretch, FontStyle, FontVariant, TextAnchor, UnicodeBidi, WritingMode, XmlLang,
+    XmlSpace,
 };
-use crate::rect::Rect;
 use crate::space::{xml_space_normalize, NormalizeDefault, XmlSpaceNormalize};
-use crate::transform::Transform;
 
 /// An absolutely-positioned array of `Span`s
 ///
@@ -273,141 +269,8 @@ impl PositionedSpan {
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
-        draw_ctx.with_saved_cr(&mut |dc| {
-            let transform = dc.get_transform();
-
-            let gravity = self.layout.get_context().unwrap().get_gravity();
-            let bbox = self.compute_text_bbox(transform, gravity);
-            if bbox.is_none() {
-                return Ok(dc.empty_bbox());
-            }
-
-            let mut bbox = if clipping {
-                dc.empty_bbox()
-            } else {
-                bbox.unwrap()
-            };
-
-            let cr = dc.get_cairo_context();
-            cr.set_antialias(cairo::Antialias::from(self.values.text_rendering()));
-            dc.setup_cr_for_stroke(&cr, &self.values);
-            cr.move_to(self.rendered_position.0, self.rendered_position.1);
-
-            let rotation = gravity.to_rotation();
-            if !rotation.approx_eq_cairo(0.0) {
-                cr.rotate(-rotation);
-            }
-
-            let current_color = self.values.color().0;
-
-            let res = if !clipping {
-                dc.set_source_paint_server(
-                    acquired_nodes,
-                    &self.values.fill().0,
-                    self.values.fill_opacity().0,
-                    &bbox,
-                    current_color,
-                )
-                .map(|had_paint_server| {
-                    if had_paint_server {
-                        pangocairo::functions::update_layout(&cr, &self.layout);
-                        pangocairo::functions::show_layout(&cr, &self.layout);
-                    };
-                })
-            } else {
-                Ok(())
-            };
-
-            if res.is_ok() {
-                let mut need_layout_path = clipping;
-
-                let res = if !clipping {
-                    dc.set_source_paint_server(
-                        acquired_nodes,
-                        &self.values.stroke().0,
-                        self.values.stroke_opacity().0,
-                        &bbox,
-                        current_color,
-                    )
-                    .map(|had_paint_server| {
-                        if had_paint_server {
-                            need_layout_path = true;
-                        }
-                    })
-                } else {
-                    Ok(())
-                };
-
-                if res.is_ok() && need_layout_path {
-                    pangocairo::functions::update_layout(&cr, &self.layout);
-                    pangocairo::functions::layout_path(&cr, &self.layout);
-
-                    if !clipping {
-                        let (x0, y0, x1, y1) = cr.stroke_extents();
-                        let r = Rect::new(x0, y0, x1, y1);
-                        let ib = BoundingBox::new()
-                            .with_transform(transform)
-                            .with_ink_rect(r);
-                        cr.stroke();
-                        bbox.insert(&ib);
-                    }
-                }
-            }
-
-            res.map(|_: ()| bbox)
-        })
-    }
-
-    fn compute_text_bbox(
-        &self,
-        transform: Transform,
-        gravity: pango::Gravity,
-    ) -> Option<BoundingBox> {
-        #![allow(clippy::many_single_char_names)]
-
-        let (ink, _) = self.layout.get_extents();
-        if ink.width == 0 || ink.height == 0 {
-            return None;
-        }
-
         let (x, y) = self.rendered_position;
-        let ink_x = f64::from(ink.x);
-        let ink_y = f64::from(ink.y);
-        let ink_width = f64::from(ink.width);
-        let ink_height = f64::from(ink.height);
-        let pango_scale = f64::from(pango::SCALE);
-
-        let (x, y, w, h) = if gravity_is_vertical(gravity) {
-            (
-                x + (ink_x - ink_height) / pango_scale,
-                y + ink_y / pango_scale,
-                ink_height / pango_scale,
-                ink_width / pango_scale,
-            )
-        } else {
-            (
-                x + ink_x / pango_scale,
-                y + ink_y / pango_scale,
-                ink_width / pango_scale,
-                ink_height / pango_scale,
-            )
-        };
-
-        let r = Rect::new(x, y, x + w, y + h);
-        let bbox = BoundingBox::new()
-            .with_transform(transform)
-            .with_rect(r)
-            .with_ink_rect(r);
-
-        Some(bbox)
-    }
-}
-
-// FIXME: should the pango crate provide this like PANGO_GRAVITY_IS_VERTICAL() ?
-fn gravity_is_vertical(gravity: pango::Gravity) -> bool {
-    match gravity {
-        pango::Gravity::East | pango::Gravity::West => true,
-        _ => false,
+        draw_ctx.draw_text(&self.layout, x, y, acquired_nodes, &self.values, clipping)
     }
 }
 
@@ -484,15 +347,16 @@ fn children_to_chunks(
 /// `xml:space="preserve"` attribute.  A `Chars` stores the characters verbatim
 /// as they come out of the XML parser, after ensuring that they are valid UTF-8.
 
+#[derive(Default)]
 pub struct Chars {
     string: RefCell<String>,
     space_normalized: RefCell<Option<String>>,
 }
 
 impl Chars {
-    pub fn new() -> Chars {
+    pub fn new(initial_text: &str) -> Chars {
         Chars {
-            string: RefCell::new(String::new()),
+            string: RefCell::new(String::from(initial_text)),
             space_normalized: RefCell::new(None),
         }
     }
@@ -564,12 +428,6 @@ impl Chars {
     }
 }
 
-impl Default for Chars {
-    fn default() -> Self {
-        Self::new()
-    }
-}
-
 #[derive(Default)]
 pub struct Text {
     x: Length<Horizontal>,
@@ -801,17 +659,6 @@ impl<'a> From<&'a XmlLang> for pango::Language {
     }
 }
 
-impl From<TextRendering> for cairo::Antialias {
-    fn from(tr: TextRendering) -> cairo::Antialias {
-        match tr {
-            TextRendering::Auto
-            | TextRendering::OptimizeLegibility
-            | TextRendering::GeometricPrecision => cairo::Antialias::Default,
-            TextRendering::OptimizeSpeed => cairo::Antialias::None,
-        }
-    }
-}
-
 impl From<FontStyle> for pango::Style {
     fn from(s: FontStyle) -> pango::Style {
         match s {
@@ -895,49 +742,12 @@ impl From<WritingMode> for pango::Gravity {
     }
 }
 
-fn get_pango_context(cr: &cairo::Context, is_testing: bool) -> pango::Context {
-    let font_map = pangocairo::FontMap::get_default().unwrap();
-    let context = font_map.create_context().unwrap();
-    pangocairo::functions::update_context(&cr, &context);
-
-    // Pango says this about pango_cairo_context_set_resolution():
-    //
-    //     Sets the resolution for the context. This is a scale factor between
-    //     points specified in a #PangoFontDescription and Cairo units. The
-    //     default value is 96, meaning that a 10 point font will be 13
-    //     units high. (10 * 96. / 72. = 13.3).
-    //
-    // I.e. Pango font sizes in a PangoFontDescription are in *points*, not pixels.
-    // However, we are normalizing everything to userspace units, which amount to
-    // pixels.  So, we will use 72.0 here to make Pango not apply any further scaling
-    // to the size values we give it.
-    //
-    // An alternative would be to divide our font sizes by (dpi_y / 72) to effectively
-    // cancel out Pango's scaling, but it's probably better to deal with Pango-isms
-    // right here, instead of spreading them out through our Length normalization
-    // code.
-    pangocairo::functions::context_set_resolution(&context, 72.0);
-
-    if is_testing {
-        let mut options = cairo::FontOptions::new();
-
-        options.set_antialias(cairo::Antialias::Gray);
-        options.set_hint_style(cairo::HintStyle::Full);
-        options.set_hint_metrics(cairo::HintMetrics::On);
-
-        pangocairo::functions::context_set_font_options(&context, Some(&options));
-    }
-
-    context
-}
-
 fn create_pango_layout(
     draw_ctx: &DrawingCtx,
     values: &ComputedValues,
     text: &str,
 ) -> pango::Layout {
-    let cr = draw_ctx.get_cairo_context();
-    let pango_context = get_pango_context(&cr, draw_ctx.is_testing());
+    let pango_context = pango::Context::from(draw_ctx);
 
     // See the construction of the XmlLang property
     // We use "" there as the default value; this means that the language is not set.
@@ -1014,3 +824,23 @@ fn create_pango_layout(
 
     layout
 }
+
+#[cfg(test)]
+mod tests {
+    use super::Chars;
+
+    #[test]
+    fn chars_default() {
+        let c = Chars::default();
+        assert!(c.is_empty());
+        assert!(c.space_normalized.borrow().is_none());
+    }
+
+    #[test]
+    fn chars_new() {
+        let example = "Test 123";
+        let c = Chars::new(example);
+        assert_eq!(c.get_string(), example);
+        assert!(c.space_normalized.borrow().is_none());
+    }
+}
